// SPDX-License-Identifier: Apache-2.0
#![deny(warnings)]

use std::convert::Infallible;

use hyper::service::{make_service_fn, service_fn};
use hyper::{Body, Request, Response, Server};

async fn hello(_: Request<Body>) -> Result<Response<Body>, Infallible> {
  Ok(Response::new(Body::from("Hello World!")))
}

async fn shutdown_signal() {
  tokio::signal::ctrl_c()
    .await
    .expect("failed to install CTRL+C handler");
}

#[tokio::main]
pub async fn main() -> Result<(), Box<dyn std::error::Error + Send + Sync>> {
  pretty_env_logger::init();
  let make_svc =
    make_service_fn(|_conn| async { Ok::<_, Infallible>(service_fn(hello)) });
  let addr = ([0, 0, 0, 0], 8080).into();
  let server = Server::bind(&addr).serve(make_svc);
  println!("Listening on http://{}", addr);
  server.with_graceful_shutdown(shutdown_signal()).await?;
  Ok(())
}
